require 'test_helper'

TicTacToe::MakeMoveCommand

class GameTest < Minitest::Test
  def setup
    @game_id = 'abcdef-0123-4567-abcdefghi'
    TicTacToe::MemoryRepository.new.create(id: @game_id)
  end

  def test_initial_board
    assert_equal [[:free, :free, :free],[:free, :free, :free],[:free, :free, :free]], game.board
    refute game.finished?
  end

  def test_making_valid_move
    make_move! player: :player_1, position: :center

    assert_equal [[:free, :free, :free],[:free, :player_1, :free],[:free, :free, :free]], game.board
  end

  def test_making_move_as_wrong_player
    exception = assert_raises TicTacToe::BadMove do
      make_move! player: :player_2, position: :center
    end
    assert_equal "It is player_1's turn", exception.message
  end

  def test_must_pick_free_space
    make_move! player: :player_1, position: :center

    exception = assert_raises TicTacToe::BadMove do
      make_move! player: :player_2, position: :center
    end
    assert_equal "Tile at center is already occupied!", exception.message
  end

  def test_three_in_a_row_ends_game
    make_move! player: :player_1, position: :left
    make_move! player: :player_2, position: :right
    make_move! player: :player_1, position: :bottom
    make_move! player: :player_2, position: :top_right
    make_move! player: :player_1, position: :bottom_left
    make_move! player: :player_2, position: :top_left
    make_move! player: :player_1, position: :bottom_right

    assert game.finished?
    assert_equal :player_1, game.winner

    exception = assert_raises TicTacToe::BadMove do
      make_move! player: :player_2, position: :top
    end
    assert_equal "Game has already ended!", exception.message
  end

  private

  def make_move!(**params)
    TicTacToe.build('MakeMoveCommand', game_id: @game_id, **params).apply!
  end

  def game
    TicTacToe::MemoryRepository.new.find @game_id
  end
end
